@@ -1,4 +1,3 @@
-/* $XdotOrg: xserver/xorg/Xext/xvmain.c,v 1.8 2006/02/15 20:44:12 ajax Exp $ */
 /***********************************************************
 Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts,
 and the Massachusetts Institute of Technology, Cambridge, Massachusetts.
@@ -22,7 +21,6 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 ******************************************************************/
-/* $XFree86: xc/programs/Xserver/Xext/xvmain.c,v 1.15tsi Exp $ */
 
 /*
 ** File: 
@@ -104,10 +102,11 @@ SOFTWARE.
 #ifdef PANORAMIX
 #include "panoramiX.h"
 #include "panoramiXsrv.h"
-#include "xvdisp.h"
 #endif
+#include "xvdisp.h"
 
-int  XvScreenIndex = -1;
+static int XvScreenKeyIndex;
+static DevPrivateKey XvScreenKey = &XvScreenKeyIndex;
 unsigned long XvExtensionGeneration = 0;
 unsigned long XvScreenGeneration = 0;
 unsigned long XvResourceGeneration = 0;
@@ -154,8 +153,8 @@ static int XvdiSendVideoNotify(XvPortPtr, DrawablePtr, int);
 **
 */
 
-void 
-XvExtensionInit()
+void
+XvExtensionInit(void)
 {
   ExtensionEntry *extEntry;
 
@@ -168,12 +167,6 @@ XvExtensionInit()
 	  ErrorF("XvExtensionInit: Unable to allocate resource types\n");
 	  return;
 	}
-      XvScreenIndex = AllocateScreenPrivateIndex ();
-      if (XvScreenIndex < 0)
-	{
-	  ErrorF("XvExtensionInit: Unable to allocate screen private index\n");
-	  return;
-	}
 #ifdef PANORAMIX
         XineramaRegisterConnectionBlockCallback(XineramifyXv);
 #endif
@@ -207,7 +200,7 @@ XvExtensionInit()
 }
 
 static Bool
-CreateResourceTypes()
+CreateResourceTypes(void)
 
 {
   
@@ -215,37 +208,41 @@ CreateResourceTypes()
 
   XvResourceGeneration = serverGeneration;
 
-  if (!(XvRTPort = CreateNewResourceType(XvdiDestroyPort)))
+  if (!(XvRTPort = CreateNewResourceType(XvdiDestroyPort, "XvRTPort")))
     {
       ErrorF("CreateResourceTypes: failed to allocate port resource.\n");
       return FALSE;
     }
-  
-  if (!(XvRTGrab = CreateNewResourceType(XvdiDestroyGrab)))
+
+  if (!(XvRTGrab = CreateNewResourceType(XvdiDestroyGrab, "XvRTGrab")))
     {
       ErrorF("CreateResourceTypes: failed to allocate grab resource.\n");
       return FALSE;
     }
-  
-  if (!(XvRTEncoding = CreateNewResourceType(XvdiDestroyEncoding)))
+
+  if (!(XvRTEncoding = CreateNewResourceType(XvdiDestroyEncoding,
+					     "XvRTEncoding")))
     {
       ErrorF("CreateResourceTypes: failed to allocate encoding resource.\n");
       return FALSE;
     }
-  
-  if (!(XvRTVideoNotify = CreateNewResourceType(XvdiDestroyVideoNotify)))
+
+  if (!(XvRTVideoNotify = CreateNewResourceType(XvdiDestroyVideoNotify,
+						"XvRTVideoNotify")))
     {
       ErrorF("CreateResourceTypes: failed to allocate video notify resource.\n");
       return FALSE;
     }
-  
-  if (!(XvRTVideoNotifyList = CreateNewResourceType(XvdiDestroyVideoNotifyList)))
+
+  if (!(XvRTVideoNotifyList = CreateNewResourceType(XvdiDestroyVideoNotifyList,
+						    "XvRTVideoNotifyList")))
     {
       ErrorF("CreateResourceTypes: failed to allocate video notify list resource.\n");
       return FALSE;
     }
 
-  if (!(XvRTPortNotify = CreateNewResourceType(XvdiDestroyPortNotify)))
+  if (!(XvRTPortNotify = CreateNewResourceType(XvdiDestroyPortNotify,
+					       "XvRTPortNotify")))
     {
       ErrorF("CreateResourceTypes: failed to allocate port notify resource.\n");
       return FALSE;
@@ -255,7 +252,7 @@ CreateResourceTypes()
 
 }
 
-_X_EXPORT int
+int
 XvScreenInit(ScreenPtr pScreen)
 {
   XvScreenPtr pxvs;
@@ -267,33 +264,27 @@ XvScreenInit(ScreenPtr pScreen)
 	  ErrorF("XvScreenInit: Unable to allocate resource types\n");
 	  return BadAlloc;
 	}
-      XvScreenIndex = AllocateScreenPrivateIndex ();
-      if (XvScreenIndex < 0)
-	{
-	  ErrorF("XvScreenInit: Unable to allocate screen private index\n");
-	  return BadAlloc;
-	}
 #ifdef PANORAMIX
         XineramaRegisterConnectionBlockCallback(XineramifyXv);
 #endif
       XvScreenGeneration = serverGeneration; 
     }
 
-  if (pScreen->devPrivates[XvScreenIndex].ptr)
+  if (dixLookupPrivate(&pScreen->devPrivates, XvScreenKey))
     {
       ErrorF("XvScreenInit: screen devPrivates ptr non-NULL before init\n");
     }
 
   /* ALLOCATE SCREEN PRIVATE RECORD */
   
-  pxvs = (XvScreenPtr) xalloc (sizeof (XvScreenRec));
+  pxvs = xalloc (sizeof (XvScreenRec));
   if (!pxvs)
     {
       ErrorF("XvScreenInit: Unable to allocate screen private structure\n");
       return BadAlloc;
     }
 
-  pScreen->devPrivates[XvScreenIndex].ptr = (pointer)pxvs;
+  dixSetPrivate(&pScreen->devPrivates, XvScreenKey, pxvs);
 
   
   pxvs->DestroyPixmap = pScreen->DestroyPixmap;
@@ -315,7 +306,7 @@ XvCloseScreen(
 
   XvScreenPtr pxvs;
 
-  pxvs = (XvScreenPtr) pScreen->devPrivates[XvScreenIndex].ptr;
+  pxvs = (XvScreenPtr)dixLookupPrivate(&pScreen->devPrivates, XvScreenKey);
 
   pScreen->DestroyPixmap = pxvs->DestroyPixmap;
   pScreen->DestroyWindow = pxvs->DestroyWindow;
@@ -325,25 +316,25 @@ XvCloseScreen(
 
   xfree(pxvs);
 
-  pScreen->devPrivates[XvScreenIndex].ptr = (pointer)NULL;
+  dixSetPrivate(&pScreen->devPrivates, XvScreenKey, NULL);
 
   return (*pScreen->CloseScreen)(ii, pScreen);
-
 }
 
 static void
 XvResetProc(ExtensionEntry* extEntry)
 {
+    XvResetProcVector();
 }
 
-_X_EXPORT int
-XvGetScreenIndex()
+DevPrivateKey
+XvGetScreenKey(void)
 {
-  return XvScreenIndex;
+    return XvScreenKey;
 }
 
-_X_EXPORT unsigned long
-XvGetRTPort()
+unsigned long
+XvGetRTPort(void)
 {
   return XvRTPort;
 }
@@ -363,7 +354,7 @@ XvDestroyPixmap(PixmapPtr pPix)
 
   SCREEN_PROLOGUE(pScreen, DestroyPixmap);
 
-  pxvs = (XvScreenPtr)pScreen->devPrivates[XvScreenIndex].ptr;
+  pxvs = (XvScreenPtr)dixLookupPrivate(&pScreen->devPrivates, XvScreenKey);
 
   /* CHECK TO SEE IF THIS PORT IS IN USE */
 
@@ -380,11 +371,10 @@ XvDestroyPixmap(PixmapPtr pPix)
 	    {
 	      XvdiSendVideoNotify(pp, pp->pDraw, XvPreempted);
 
-	      (void)(* pp->pAdaptor->ddStopVideo)((ClientPtr)NULL, pp, 
-						  pp->pDraw);
+	      (void)(* pp->pAdaptor->ddStopVideo)(NULL, pp, pp->pDraw);
 
-	      pp->pDraw = (DrawablePtr)NULL;
-	      pp->client = (ClientPtr)NULL;
+	      pp->pDraw = NULL;
+	      pp->client = NULL;
 	      pp->time = currentTime;
 	    }
 	  pp++;
@@ -415,7 +405,7 @@ XvDestroyWindow(WindowPtr pWin)
 
   SCREEN_PROLOGUE(pScreen, DestroyWindow);
 
-  pxvs = (XvScreenPtr)pScreen->devPrivates[XvScreenIndex].ptr;
+  pxvs = (XvScreenPtr)dixLookupPrivate(&pScreen->devPrivates, XvScreenKey);
 
   /* CHECK TO SEE IF THIS PORT IS IN USE */
 
@@ -432,11 +422,10 @@ XvDestroyWindow(WindowPtr pWin)
 	    {
 	      XvdiSendVideoNotify(pp, pp->pDraw, XvPreempted);
 
-	      (void)(* pp->pAdaptor->ddStopVideo)((ClientPtr)NULL, pp, 
-						  pp->pDraw);
+	      (void)(* pp->pAdaptor->ddStopVideo)(NULL, pp, pp->pDraw);
 
-	      pp->pDraw = (DrawablePtr)NULL;
-	      pp->client = (ClientPtr)NULL;
+	      pp->pDraw = NULL;
+	      pp->client = NULL;
 	      pp->time = currentTime;
 	    }
 	  pp++;
@@ -468,8 +457,8 @@ XvdiVideoStopped(XvPortPtr pPort, int reason)
 
   XvdiSendVideoNotify(pPort, pPort->pDraw, reason);
 
-  pPort->pDraw = (DrawablePtr)NULL;
-  pPort->client = (ClientPtr)NULL;
+  pPort->pDraw = NULL;
+  pPort->client = NULL;
   pPort->time = currentTime;
 
   return Success;
@@ -485,7 +474,7 @@ XvdiDestroyPort(pointer pPort, XID id)
 static int
 XvdiDestroyGrab(pointer pGrab, XID id)
 {
-  ((XvGrabPtr)pGrab)->client = (ClientPtr)NULL;
+  ((XvGrabPtr)pGrab)->client = NULL;
   return Success;
 }
 
@@ -494,7 +483,7 @@ XvdiDestroyVideoNotify(pointer pn, XID id)
 {
   /* JUST CLEAR OUT THE client POINTER FIELD */
 
-  ((XvVideoNotifyPtr)pn)->client = (ClientPtr)NULL;
+  ((XvVideoNotifyPtr)pn)->client = NULL;
   return Success;
 }
 
@@ -503,7 +492,7 @@ XvdiDestroyPortNotify(pointer pn, XID id)
 {
   /* JUST CLEAR OUT THE client POINTER FIELD */
 
-  ((XvPortNotifyPtr)pn)->client = (ClientPtr)NULL;
+  ((XvPortNotifyPtr)pn)->client = NULL;
   return Success;
 }
 
@@ -533,17 +522,13 @@ XvdiDestroyEncoding(pointer value, XID id)
 }
 
 static int
-XvdiSendVideoNotify(pPort, pDraw, reason)
-
-XvPortPtr pPort;
-DrawablePtr pDraw;
-int reason;
-
+XvdiSendVideoNotify(XvPortPtr pPort, DrawablePtr pDraw, int reason)
 {
   xvEvent event;
   XvVideoNotifyPtr pn;
 
-  pn = (XvVideoNotifyPtr)LookupIDByType(pDraw->id, XvRTVideoNotifyList);
+  dixLookupResourceByType((pointer *)&pn, pDraw->id, XvRTVideoNotifyList,
+			  serverClient, DixReadAccess);
 
   while (pn) 
     {
@@ -555,8 +540,8 @@ int reason;
 	  event.u.videoNotify.drawable = pDraw->id;
 	  event.u.videoNotify.port = pPort->id;
 	  event.u.videoNotify.reason = reason;
-	  (void) TryClientEvents(pn->client, (xEventPtr)&event, 1, NoEventMask,
-				 NoEventMask, NullGrab);
+	  TryClientEvents(pn->client, NULL, (xEventPtr)&event, 1,
+                          NoEventMask, NoEventMask, NullGrab);
 	}
       pn = pn->next;
     }
@@ -587,8 +572,8 @@ XvdiSendPortNotify(
 	  event.u.portNotify.port = pPort->id;
 	  event.u.portNotify.attribute = attribute;
 	  event.u.portNotify.value = value;
-	  (void) TryClientEvents(pn->client, (xEventPtr)&event, 1, NoEventMask,
-				 NoEventMask, NullGrab);
+	  TryClientEvents(pn->client, NULL, (xEventPtr)&event, 1,
+                          NoEventMask, NoEventMask, NullGrab);
 	}
       pn = pn->next;
     }
@@ -870,7 +855,7 @@ XvdiGrabPort(
 
   if ((pPort->pDraw) && (client != pPort->client))
     {
-      XVCALL(diStopVideo)((ClientPtr)NULL, pPort, pPort->pDraw);
+      XvdiStopVideo(NULL, pPort, pPort->pDraw);
     }
 
   pPort->grab.client = client;
@@ -909,7 +894,7 @@ XvdiUngrabPort(
   /* FREE THE GRAB RESOURCE; AND SET THE GRAB CLIENT TO NULL */
 
   FreeResource(pPort->grab.id, XvRTGrab);
-  pPort->grab.client = (ClientPtr)NULL;
+  pPort->grab.client = NULL;
 
   pPort->time = currentTime;
 
@@ -925,10 +910,14 @@ XvdiSelectVideoNotify(
   BOOL onoff
 ){
   XvVideoNotifyPtr pn,tpn,fpn;
+  int rc;
 
   /* FIND VideoNotify LIST */
 
-  pn = (XvVideoNotifyPtr)LookupIDByType(pDraw->id, XvRTVideoNotifyList);
+  rc = dixLookupResourceByType((pointer *)&pn, pDraw->id, XvRTVideoNotifyList,
+			       client, DixWriteAccess);
+  if (rc != Success && rc != BadValue)
+      return rc;
 
   /* IF ONE DONES'T EXIST AND NO MASK, THEN JUST RETURN */
 
@@ -939,9 +928,9 @@ XvdiSelectVideoNotify(
 
   if (!pn) 
     {
-      if (!(tpn = (XvVideoNotifyPtr)xalloc(sizeof(XvVideoNotifyRec))))
+      if (!(tpn = xalloc(sizeof(XvVideoNotifyRec))))
 	return BadAlloc;
-      tpn->next = (XvVideoNotifyPtr)NULL;
+      tpn->next = NULL;
       if (!AddResource(pDraw->id, XvRTVideoNotifyList, tpn))
 	{
 	  xfree(tpn);
@@ -952,13 +941,13 @@ XvdiSelectVideoNotify(
     {
       /* LOOK TO SEE IF ENTRY ALREADY EXISTS */
 
-      fpn = (XvVideoNotifyPtr)NULL;
+      fpn = NULL;
       tpn = pn;
       while (tpn)
 	{
 	  if (tpn->client == client) 
 	    {
-	      if (!onoff) tpn->client = (ClientPtr)NULL;
+	      if (!onoff) tpn->client = NULL;
 	      return Success;
 	    }
 	  if (!tpn->client) fpn = tpn; /* TAKE NOTE OF FREE ENTRY */
@@ -977,7 +966,7 @@ XvdiSelectVideoNotify(
 	}
       else
 	{
-	  if (!(tpn = (XvVideoNotifyPtr)xalloc(sizeof(XvVideoNotifyRec))))
+	  if (!(tpn = xalloc(sizeof(XvVideoNotifyRec))))
 	    return BadAlloc;
 	  tpn->next = pn->next;
 	  pn->next = tpn;
@@ -987,7 +976,7 @@ XvdiSelectVideoNotify(
   /* INIT CLIENT PTR IN CASE WE CAN'T ADD RESOURCE */
   /* ADD RESOURCE SO THAT IF CLIENT EXITS THE CLIENT PTR WILL BE CLEARED */
 
-  tpn->client = (ClientPtr)NULL;
+  tpn->client = NULL;
   tpn->id = FakeClientID(client->index);
   AddResource(tpn->id, XvRTVideoNotify, tpn);
 
@@ -1006,7 +995,7 @@ XvdiSelectPortNotify(
 
   /* SEE IF CLIENT IS ALREADY IN LIST */
 
-  tpn = (XvPortNotifyPtr)NULL;
+  tpn = NULL;
   pn = pPort->pNotify;
   while (pn)
     {
@@ -1023,7 +1012,7 @@ XvdiSelectPortNotify(
 
       if (!onoff)
 	{
-	  pn->client = (ClientPtr)NULL;
+	  pn->client = NULL;
 	  FreeResource(pn->id, XvRTPortNotify);
 	}
 
@@ -1035,7 +1024,7 @@ XvdiSelectPortNotify(
 
   if (!tpn)
     {
-      if (!(tpn = (XvPortNotifyPtr)xalloc(sizeof(XvPortNotifyRec))))
+      if (!(tpn = xalloc(sizeof(XvPortNotifyRec))))
 	return BadAlloc;
       tpn->next = pPort->pNotify;
       pPort->pNotify = tpn;
@@ -1078,7 +1067,7 @@ XvdiStopVideo(
 
   status = (* pPort->pAdaptor->ddStopVideo)(client, pPort, pDraw);
 
-  pPort->pDraw = (DrawablePtr)NULL;
+  pPort->pDraw = NULL;
   pPort->client = (ClientPtr)client;
   pPort->time = currentTime;
 
@@ -1102,7 +1091,7 @@ XvdiPreemptVideo(
 
   status = (* pPort->pAdaptor->ddStopVideo)(client, pPort, pPort->pDraw);
 
-  pPort->pDraw = (DrawablePtr)NULL;
+  pPort->pDraw = NULL;
   pPort->client = (ClientPtr)client;
   pPort->time = currentTime;
 
